@@ -7,12 +7,11 @@ use keeshond_treats::visual::{Costume, Animation, AnimationCel, SliceHandle, Ani
 
 use crate::document::{Document, DocumentState, ActionHistory};
 use crate::editorgui::{ResourceType, Packages, EditorConfig, ProjectConfig};
-use crate::util::zoom::ZoomControl;
+use crate::util::zoom::{EDITOR_ZOOM_LEVELS, ZoomControl};
 
 use std::any::Any;
 
 const CEL_BUTTON_SIZE : f32 = 80.0;
-const ZOOM_LEVELS : [f32 ; 11] = [0.25, 0.5, 1.0, 2.0, 3.0, 4.0, 6.0, 8.0, 10.0, 12.0, 16.0];
 
 enum CostumeAction
 {
@@ -739,7 +738,7 @@ impl Document for CostumeDocument
             data_id,
             show_origin : true,
             transparent_color : [0.5, 0.5001, 0.5002],
-            zoom_control : ZoomControl::new(ZOOM_LEVELS.to_vec(), (1.0 * dpi_factor).round()),
+            zoom_control : ZoomControl::new(EDITOR_ZOOM_LEVELS.to_vec(), (1.0 * dpi_factor).round()),
             dpi_factor,
             current_animation : 0,
             current_frame : 0,
@@ -930,7 +929,7 @@ impl Document for CostumeDocument
             });
             ui.menu("Zoom", ||
             {
-                for level in &ZOOM_LEVELS
+                for level in &EDITOR_ZOOM_LEVELS
                 {
                     if imgui::MenuItem::new(format!("{}%", (level * 100.0) as i32)).selected(self.zoom_control.zoom() == *level).build(ui)
                     {
@@ -1113,8 +1112,8 @@ impl Document for CostumeDocument
                             let slice_space = ui.window_size();
                             let center_x = self.scaled_round(slice_space[0] / 2.0);
                             let center_y = self.scaled_round(slice_space[1] / 2.0);
-                            let image_x = self.scaled_round(center_x - (rect.x - cel.x) * self.adjusted_zoom());
-                            let image_y = self.scaled_round(center_y - (rect.y - cel.y) * self.adjusted_zoom());
+                            let image_x = self.scaled_round(center_x - rect.x * self.adjusted_zoom());
+                            let image_y = self.scaled_round(center_y - rect.y * self.adjusted_zoom());
 
                             ui.set_cursor_pos([image_x, image_y]);
                             crate::util::widget::round_cursor_screen_pos(ui);
