@@ -9,10 +9,9 @@ use keeshond::datapack::DataId;
 use std::any::Any;
 
 use image::{RgbaImage, GrayImage};
-use crate::util::zoom::ZoomControl;
+use crate::util::zoom::{EDITOR_ZOOM_LEVELS, ZoomControl};
 
 const MAX_DRAW_POINTS : usize = 32768;
-const ZOOM_LEVELS : [f32 ; 11] = [0.25, 0.5, 1.0, 2.0, 3.0, 4.0, 6.0, 8.0, 10.0, 12.0, 16.0];
 
 enum SheetAction
 {
@@ -433,13 +432,7 @@ impl SheetDocument
             }
         });
 
-        ui.same_line();
-        ui.text_disabled("(?)");
-
-        if ui.is_item_hovered()
-        {
-            ui.tooltip_text("Use this if you see an undesired halo around the edges of your graphics\nwhen zooming and texture filtering is enabled.");
-        }
+        crate::util::widget::hover_hint("Use this if you see an undesired halo around the edges of your graphics\nwhen zooming and texture filtering is enabled.", ui);
 
         ui.separator();
         ui.spacing();
@@ -883,7 +876,7 @@ impl Document for SheetDocument
             data_id,
             show_origin : true,
             transparent_color : [0.5, 0.5001, 0.5002],
-            zoom_control : ZoomControl::new(ZOOM_LEVELS.to_vec(), (1.0 * dpi_factor).round()),
+            zoom_control : ZoomControl::new(EDITOR_ZOOM_LEVELS.to_vec(), (1.0 * dpi_factor).round()),
             dpi_factor,
             current_slice : 0,
             tics : 0,
@@ -1059,7 +1052,7 @@ impl Document for SheetDocument
             });
             ui.menu("Zoom", ||
             {
-                for level in &ZOOM_LEVELS
+                for level in &EDITOR_ZOOM_LEVELS
                 {
                     if imgui::MenuItem::new(format!("{}%", (level * 100.0) as i32))
                         .selected(self.zoom_control.zoom() == *level).build(ui)
