@@ -14,7 +14,7 @@ use keeshond_treats::LoadErrorMode;
 use keeshond_treats::visual::{Costume, DrawableDrawer, Drawable, ActorThinker, PixelSnap};
 use keeshond_treats::tilemap::Tileset;
 use keeshond_treats::spawnable::SpawnableSet;
-use keeshond_treats::world::Level;
+use keeshond_treats::level::Level;
 
 use std::collections::{HashMap, HashSet};
 use std::string::ToString;
