@@ -1,4 +1,6 @@
 use std::collections::{BTreeMap, BTreeSet, HashMap};
+use std::fs::File;
+use std::io::BufReader;
 use std::path::{Component, PathBuf};
 use std::sync::mpsc::{channel, Receiver};
 use std::time::{Duration, Instant};
@@ -7,6 +9,7 @@ use chrono::{DateTime, TimeZone, Utc};
 use notify::{DebouncedEvent, RecommendedWatcher, RecursiveMode, Watcher};
 use path_slash::{PathBufExt, PathExt};
 use serde::{Deserialize, Serialize};
+use sha2::{Digest, Sha256};
 use walkdir::WalkDir;
 
 use keeshond::gameloop::GameControl;
@@ -22,12 +25,14 @@ use crate::import::sheet::{BackgroundImportData, ImportedImage, SpriteImportData
 pub mod sheet;
 
 const REGISTRY_CONFIG_FILENAME : &str = "Registry.toml";
+const LOCAL_REGISTRY_CONFIG_FILENAME : &str = "Registry.local.toml";
 const DEFAULT_MAX_TEXTURE_SIZE : u32 = 4096;
 const DEFAULT_PADDING : u32 = 1;
 
-const EXCLUDED_FILENAMES : [&str; 7] =
+const EXCLUDED_FILENAMES : [&str; 8] =
 [
     REGISTRY_CONFIG_FILENAME,
+    LOCAL_REGISTRY_CONFIG_FILENAME,
     "desktop.ini",
     "Thumbs.db",
     ".DS_Store",
@@ -86,7 +91,7 @@ impl ImportItemPath
 pub struct ImportConfig
 {
     #[serde(flatten)]
-    entries : HashMap<String, String>
+    entries : BTreeMap<String, String>
 }
 
 impl ImportConfig
@@ -95,7 +100,7 @@ impl ImportConfig
     {
         ImportConfig
         {
-            entries : HashMap::new()
+            entries : BTreeMap::new()
         }
     }
 
@@ -177,6 +182,32 @@ pub enum ImporterResult
     Err(String)
 }
 
+#[derive(Copy, Clone, PartialEq)]
+enum RegistryEntryUpdateType
+{
+    None,
+    Ok
+    {
+        sha2 : [u8; 32]
+    },
+    Deferred,
+    NeedsConfig,
+    UpdateTimestamp
+}
+
+impl RegistryEntryUpdateType
+{
+    fn is_ok(&self) -> bool
+    {
+        if let RegistryEntryUpdateType::Ok { .. } = self
+        {
+            return true;
+        }
+
+        false
+    }
+}
+
 pub trait Importer
 {
     fn name(&self) -> &'static str;
@@ -190,6 +221,7 @@ pub trait Importer
 pub struct ImportGroupItem<T>
 {
     pathbuf : PathBuf,
+    sha2 : [u8; 32],
     data : T
 }
 
@@ -210,10 +242,13 @@ enum ProcessImportResult
 #[derive(Clone, Serialize, Deserialize)]
 struct RegistryEntry
 {
-    last_import : DateTime<Utc>,
     #[serde(default)]
     importer : Option<String>,
     #[serde(default)]
+    auto_import_on_change : bool,
+    #[serde(default)]
+    last_sha2 : Option<String>,
+    #[serde(default)]
     config : Option<ImportConfig>
 }
 
@@ -223,9 +258,27 @@ impl Default for RegistryEntry
     {
         RegistryEntry
         {
-            last_import : Utc.ymd(1970, 1, 1).and_hms(0, 0, 0),
             importer : None,
-            config : None
+            config : None,
+            auto_import_on_change : false,
+            last_sha2 : None
+        }
+    }
+}
+
+#[derive(Clone, Serialize, Deserialize)]
+struct LocalRegistryEntry
+{
+    last_import : DateTime<Utc>
+}
+
+impl Default for LocalRegistryEntry
+{
+    fn default() -> Self
+    {
+        LocalRegistryEntry
+        {
+            last_import : Utc.ymd(1970, 1, 1).and_hms(0, 0, 0)
         }
     }
 }
@@ -263,11 +316,20 @@ fn valid_import_file(pathbuf : &PathBuf) -> bool
     pathbuf.is_file() && !EXCLUDED_FILENAMES.contains(&&*pathbuf.file_name().unwrap_or_default().to_string_lossy().to_string())
 }
 
-fn add_auto_pending(changed_files : &mut BTreeSet<PathBuf>, pathbuf : PathBuf) -> bool
+fn add_auto_pending(registry : &BTreeMap<String, RegistryEntry>, import_directory : &String,
+                    changed_files : &mut BTreeSet<ChangedFileInfo>, changed_file_info : ChangedFileInfo) -> bool
 {
-    if valid_import_file(&pathbuf)
+    let registry_path = filepath_for_registry(changed_file_info.pathbuf.clone(), import_directory);
+    let mut auto_import = true;
+
+    if let Some(entry) = registry.get(&registry_path)
     {
-        changed_files.insert(pathbuf);
+        auto_import = entry.auto_import_on_change;
+    }
+
+    if auto_import && valid_import_file(&changed_file_info.pathbuf)
+    {
+        changed_files.insert(changed_file_info);
         return true;
     }
 
@@ -308,16 +370,33 @@ impl ImportProcessingState
     }
 }
 
+enum FileHashInfo
+{
+    Modified
+    {
+        sha2 : [u8; 32]
+    },
+    Unmodified
+}
+
+#[derive(Ord, PartialOrd, Eq, PartialEq)]
+struct ChangedFileInfo
+{
+    pathbuf : PathBuf,
+    ignore_same_hash : bool
+}
+
 pub struct AutoImport
 {
     import_directory : String,
     max_texture_size : u32,
     notifier : Option<AutoImportNotifier>,
-    changed_files : BTreeSet<PathBuf>,
+    changed_files : BTreeSet<ChangedFileInfo>,
     needs_config_files : BTreeSet<PathBuf>,
     configured_files : BTreeSet<PathBuf>,
     configured_file_names : Vec<String>,
-    registry : HashMap<String, RegistryEntry>,
+    registry : BTreeMap<String, RegistryEntry>,
+    local_registry : BTreeMap<String, LocalRegistryEntry>,
     importers : HashMap<String, Box<dyn Importer>>,
     importer_extensions : HashMap<String, Vec<String>>,
     ui_current_item : Option<CurrentItem>
@@ -336,7 +415,8 @@ impl AutoImport
             needs_config_files : BTreeSet::new(),
             configured_files : BTreeSet::new(),
             configured_file_names : Vec::new(),
-            registry : HashMap::new(),
+            registry : BTreeMap::new(),
+            local_registry : BTreeMap::new(),
             importers : HashMap::new(),
             importer_extensions : HashMap::new(),
             ui_current_item : None
@@ -383,18 +463,20 @@ impl AutoImport
 
     fn load_registry(&mut self)
     {
-        self.registry = confy::load_path(format!("{}/{}", self.import_directory, REGISTRY_CONFIG_FILENAME)).unwrap_or(HashMap::new());
+        self.registry = confy::load_path(format!("{}/{}", self.import_directory, REGISTRY_CONFIG_FILENAME)).unwrap_or(BTreeMap::new());
+        self.local_registry = confy::load_path(format!("{}/{}", self.import_directory, LOCAL_REGISTRY_CONFIG_FILENAME)).unwrap_or(BTreeMap::new());
     }
 
     fn save_registry(&self)
     {
-        match confy::store_path(format!("{}/{}", self.import_directory, REGISTRY_CONFIG_FILENAME), &self.registry)
+        if let Err(error) = confy::store_path(format!("{}/{}", self.import_directory, REGISTRY_CONFIG_FILENAME), &self.registry)
         {
-            Ok(_) => {},
-            Err(error) =>
-            {
-                error!("Error while saving AutoImport registry: {}", error);
-            }
+            error!("Error while saving AutoImport registry: {}", error);
+        }
+
+        if let Err(error) = confy::store_path(format!("{}/{}", self.import_directory, LOCAL_REGISTRY_CONFIG_FILENAME), &self.local_registry)
+        {
+            error!("Error while saving local AutoImport registry: {}", error);
         }
     }
 
@@ -429,9 +511,12 @@ impl AutoImport
                     {
                         match event
                         {
-                            DebouncedEvent::Create(path) => { add_auto_pending(&mut self.changed_files, path); }
-                            DebouncedEvent::Write(path) => { add_auto_pending(&mut self.changed_files, path); }
-                            DebouncedEvent::Rename(_, path) => { add_auto_pending(&mut self.changed_files, path); }
+                            DebouncedEvent::Create(path) => { add_auto_pending(&self.registry, &self.import_directory, &mut self.changed_files,
+                                                                              ChangedFileInfo { pathbuf : path, ignore_same_hash : false }); }
+                            DebouncedEvent::Write(path) => { add_auto_pending(&self.registry, &self.import_directory, &mut self.changed_files,
+                                                                              ChangedFileInfo { pathbuf : path, ignore_same_hash : false }); }
+                            DebouncedEvent::Rename(_, path) => { add_auto_pending(&self.registry, &self.import_directory, &mut self.changed_files,
+                                                                              ChangedFileInfo { pathbuf : path, ignore_same_hash : false }); }
                             _ => {}
                         }
                     },
@@ -461,7 +546,24 @@ impl AutoImport
         }
     }
 
-    fn process_import(item : ImportItem, pathbuf : PathBuf, state : &mut ImportProcessingState,
+    fn check_sha2(entry : &RegistryEntry, changed_file : &ChangedFileInfo) -> std::io::Result<FileHashInfo>
+    {
+        let file = File::open(&changed_file.pathbuf)?;
+        let mut reader = BufReader::new(file);
+
+        let mut hasher = Sha256::default();
+        std::io::copy(&mut reader, &mut hasher)?;
+        let sha2 = hasher.finalize().into();
+
+        if changed_file.ignore_same_hash || hex::encode(sha2) != entry.last_sha2.clone().unwrap_or_default()
+        {
+            return Ok(FileHashInfo::Modified { sha2 });
+        }
+
+        Ok(FileHashInfo::Unmodified)
+    }
+
+    fn process_import(item : ImportItem, pathbuf : PathBuf, sha2 : [u8; 32], state : &mut ImportProcessingState,
                        packages : &mut Packages, game : &mut GameControl) -> ProcessImportResult
     {
         match item
@@ -484,6 +586,7 @@ impl AutoImport
                 entry.push(ImportGroupItem::<SpriteImportData>
                 {
                     pathbuf,
+                    sha2,
                     data : sprite_info
                 });
 
@@ -573,33 +676,46 @@ impl AutoImport
         Ok(())
     }
 
-    fn update_registry_entry(&mut self, pathbuf : &PathBuf, success : bool)
+    fn update_registry_entry(&mut self, pathbuf : &PathBuf, update_type : RegistryEntryUpdateType)
     {
         let registry_path = filepath_for_registry(pathbuf.clone(), &self.import_directory);
-        let mut entry = self.registry.entry(registry_path.clone()).or_insert(RegistryEntry::default());
 
-        if success
+        match update_type
         {
-            entry.last_import = Utc::now();
+            RegistryEntryUpdateType::Ok { sha2 } =>
+            {
+                let mut entry = self.registry.entry(registry_path.clone()).or_insert(RegistryEntry::default());
+                let mut local_entry = self.local_registry.entry(registry_path.clone()).or_insert(LocalRegistryEntry::default());
 
-            self.configured_files.insert(pathbuf.clone());
-            self.needs_config_files.remove(pathbuf);
-            self.configured_file_names.clear();
-        }
-        else
-        {
-            self.needs_config_files.insert(pathbuf.clone());
-            self.configured_files.remove(pathbuf);
-            self.configured_file_names.clear();
+                entry.last_sha2 = Some(hex::encode(sha2));
+                entry.auto_import_on_change = true;
+                local_entry.last_import = Utc::now();
+
+                self.configured_files.insert(pathbuf.clone());
+                self.needs_config_files.remove(pathbuf);
+                self.configured_file_names.clear();
+            }
+            RegistryEntryUpdateType::UpdateTimestamp =>
+            {
+                let mut local_entry = self.local_registry.entry(registry_path.clone()).or_insert(LocalRegistryEntry::default());
+
+                local_entry.last_import = Utc::now();
+            }
+            RegistryEntryUpdateType::NeedsConfig =>
+            {
+                self.needs_config_files.insert(pathbuf.clone());
+                self.configured_files.remove(pathbuf);
+                self.configured_file_names.clear();
+            }
+            _ => {}
         }
     }
 
-    fn import_item(&mut self, pathbuf : &PathBuf, state : &mut ImportProcessingState, packages : &mut Packages, game : &mut GameControl)
+    fn import_item(&mut self, changed_file : &ChangedFileInfo, state : &mut ImportProcessingState, packages : &mut Packages, game : &mut GameControl)
     {
-        let mut success = false;
-        let mut deferred = false;
-        let registry_path = filepath_for_registry(pathbuf.clone(), &self.import_directory);
-        let path_info = ImportItemPath::from_absolute(pathbuf.clone(), &self.import_directory);
+        let mut update_type = RegistryEntryUpdateType::None;
+        let registry_path = filepath_for_registry(changed_file.pathbuf.clone(), &self.import_directory);
+        let path_info = ImportItemPath::from_absolute(changed_file.pathbuf.clone(), &self.import_directory);
 
         info!("Importing item \"{}\"", registry_path);
 
@@ -611,27 +727,43 @@ impl AutoImport
         {
             if let Some(importer) = self.importers.get_mut(importer_name)
             {
-                match importer.import(&path_info, entry.config.as_mut())
+                match AutoImport::check_sha2(entry, changed_file)
                 {
-                    ImporterResult::Ok(item) =>
+                    Ok(FileHashInfo::Modified { sha2 }) =>
                     {
-                        match AutoImport::process_import(item, pathbuf.clone(), state, packages, game)
+                        match importer.import(&path_info, entry.config.as_mut())
                         {
-                            ProcessImportResult::Ok => { success = true; }
-                            ProcessImportResult::Deferred => { deferred = true; }
-                            ProcessImportResult::Err(error) =>
+                            ImporterResult::Ok(item) =>
                             {
-                                error!("Error importing \"{}\": {}", registry_path, error);
+                                match AutoImport::process_import(item, changed_file.pathbuf.clone(), sha2, state, packages, game)
+                                {
+                                    ProcessImportResult::Ok => { update_type = RegistryEntryUpdateType::Ok { sha2 }; }
+                                    ProcessImportResult::Deferred => { update_type = RegistryEntryUpdateType::Deferred; }
+                                    ProcessImportResult::Err(error) =>
+                                    {
+                                        error!("Error importing \"{}\": {}", registry_path, error);
+                                    }
+                                }
+                            }
+                            ImporterResult::NeedsConfig =>
+                            {
+                                info!("{} Importer needs additional configuration for \"{}\"", importer_name, registry_path);
+                                update_type = RegistryEntryUpdateType::NeedsConfig;
+                            }
+                            ImporterResult::Err(error) =>
+                            {
+                                error!("{} Importer: error importing \"{}\": {}", importer_name, registry_path, error);
                             }
                         }
                     }
-                    ImporterResult::NeedsConfig =>
+                    Ok(FileHashInfo::Unmodified) =>
                     {
-                        info!("{} Importer needs additional configuration for \"{}\"", importer_name, registry_path);
-                    }
-                    ImporterResult::Err(error) =>
+                        info!("File \"{}\" is unchanged, not importing", registry_path);
+                        update_type = RegistryEntryUpdateType::UpdateTimestamp;
+                    },
+                    Err(error) =>
                     {
-                        error!("{} Importer: error importing \"{}\": {}", importer_name, registry_path, error);
+                        error!("Unable to read hash for \"{}\": {}", registry_path, error);
                     }
                 }
             }
@@ -643,14 +775,12 @@ impl AutoImport
         else
         {
             info!("Additional configuration needed for \"{}\"", registry_path);
+            update_type = RegistryEntryUpdateType::NeedsConfig;
         }
 
-        if !deferred
-        {
-            self.update_registry_entry(pathbuf, success);
-        }
+        self.update_registry_entry(&changed_file.pathbuf, update_type);
 
-        if success || deferred
+        if update_type.is_ok() || update_type == RegistryEntryUpdateType::Deferred
         {
             let import_time = (Instant::now() - import_start).as_millis();
 
@@ -772,6 +902,7 @@ impl AutoImport
                            packages : &mut Packages, game : &mut GameControl)
     {
         let mut success = false;
+        let mut update_types = Vec::new();
 
         info!("Handling shared sheet for \"{}/{}\" with {} new or changed source item(s)", package, name, group.len());
 
@@ -779,17 +910,32 @@ impl AutoImport
 
         match self.do_import_shared_sheet(package, name, group, packages, game)
         {
+            Ok(_) =>
+            {
+                for item in group
+                {
+                    update_types.push(RegistryEntryUpdateType::Ok
+                    {
+                        sha2 : item.sha2.clone()
+                    });
+                }
 
-            Ok(_) => { success = true; }
+                success = true;
+            }
             Err(error) =>
             {
+                for _ in 0..group.len()
+                {
+                    update_types.push(RegistryEntryUpdateType::None);
+                }
+
                 error!("Could not update shared sheet \"{}/{}\": {}", package, name, error);
             }
         }
 
-        for item in group
+        for i in 0..group.len()
         {
-            self.update_registry_entry(&item.pathbuf, success);
+            self.update_registry_entry(&group[i].pathbuf, update_types[i]);
         }
 
         if success
@@ -808,9 +954,9 @@ impl AutoImport
 
         let mut state = ImportProcessingState::new();
 
-        for path in &changed_files
+        for changed_file in &changed_files
         {
-            self.import_item(path, &mut state, packages, game);
+            self.import_item(changed_file, &mut state, packages, game);
         }
 
         for ((package, path), group) in &state.deferred_sheets
@@ -851,15 +997,20 @@ impl AutoImport
                     if let Ok(modified) = metadata.modified()
                     {
                         let import_time : DateTime<Utc> = DateTime::from(modified);
-                        let entry = self.registry.entry(registry_path).or_insert(RegistryEntry::default());
+                        let entry = self.registry.entry(registry_path.clone()).or_insert(RegistryEntry::default());
+                        let local_entry = self.local_registry.entry(registry_path).or_insert(LocalRegistryEntry::default());
 
-                        if force_old_files || import_time > entry.last_import
+                        if force_old_files || entry.auto_import_on_change && import_time > local_entry.last_import
                         {
-                            self.changed_files.insert(pathbuf);
+                            self.changed_files.insert(ChangedFileInfo { pathbuf, ignore_same_hash : force_old_files });
+                        }
+                        else if entry.config.is_some()
+                        {
+                            self.configured_files.insert(pathbuf);
                         }
                         else
                         {
-                            self.configured_files.insert(pathbuf);
+                            self.needs_config_files.insert(pathbuf);
                         }
                     }
                 }
@@ -1088,7 +1239,8 @@ impl AutoImport
 
                 if ui.button("Import Now")
                 {
-                    if !add_auto_pending(&mut self.changed_files, path.clone())
+                    if !add_auto_pending(&self.registry, &self.import_directory, &mut self.changed_files,
+                                         ChangedFileInfo { pathbuf : path.clone(), ignore_same_hash : true })
                     {
                         let registry_path = filepath_for_registry(path.clone(), &self.import_directory);
                         error!("\"{}\" is not a valid import file.", registry_path);
@@ -1153,7 +1305,7 @@ impl AutoImport
                     ui.spacing();
                     ui.spacing();
 
-                    let mut registry = HashMap::new();
+                    let mut registry = BTreeMap::new();
 
                     std::mem::swap(&mut self.registry, &mut registry);
 
