@@ -44,27 +44,27 @@ impl Ord for ImportedImage
         match self
         {
             ImportedImage::Rgb(image) =>
+            {
+                match other
                 {
-                    match other
+                    ImportedImage::Rgb(other_image) =>
                     {
-                        ImportedImage::Rgb(other_image) =>
-                            {
-                                image.as_raw().cmp(other_image.as_raw())
-                            }
-                        ImportedImage::Indexed(_) => { Ordering::Greater }
+                        image.as_raw().cmp(other_image.as_raw())
                     }
+                    ImportedImage::Indexed(_) => { Ordering::Greater }
                 }
+            }
             ImportedImage::Indexed(image) =>
+            {
+                match other
                 {
-                    match other
+                    ImportedImage::Rgb(_) => { Ordering::Less }
+                    ImportedImage::Indexed(other_image) =>
                     {
-                        ImportedImage::Rgb(_) => { Ordering::Less }
-                        ImportedImage::Indexed(other_image) =>
-                            {
-                                image.as_raw().cmp(other_image.as_raw())
-                            }
+                        image.as_raw().cmp(other_image.as_raw())
                     }
                 }
+            }
         }
     }
 }
@@ -146,10 +146,10 @@ impl ImportedImage
         {
             return keeshond::util::Rect
             {
-                x : 0,
-                y : 0,
-                w : width,
-                h : height
+                x : width / 2,
+                y : height / 2,
+                w : width.min(2),
+                h : height.min(2)
             };
         }
 
@@ -169,9 +169,9 @@ impl ImportedImage
             ImportedImage::Rgb(image) =>
             {
                 let rect = ImportedImage::crop_dimensions(image, |pixel|
-                    {
-                        pixel.0[3] == 0
-                    });
+                {
+                    pixel.0[3] == 0
+                });
 
                 let image_width = rect.w + padding * 2;
                 let image_height = rect.h + padding * 2;
@@ -181,7 +181,7 @@ impl ImportedImage
 
                 new_image.copy_from(&cropped, padding, padding).unwrap();
 
-                crate::util::defringe_image(&mut new_image, false);
+                crate::util::sheet::defringe_image(&mut new_image, false);
 
                 *image = new_image;
 
@@ -190,9 +190,9 @@ impl ImportedImage
             ImportedImage::Indexed(image) =>
             {
                 let rect = ImportedImage::crop_dimensions(image, |pixel|
-                    {
-                        pixel.0[0] == 0
-                    });
+                {
+                    pixel.0[0] == 0
+                });
 
                 let image_width = rect.w + 2;
                 let image_height = rect.h + 2;
@@ -217,15 +217,21 @@ pub struct ImageImportData
     pub origin_y : f32
 }
 
+pub struct FrameTrackData
+{
+    pub track_name : String,
+    pub index : usize
+}
+
 pub struct FrameImportData
 {
-    pub index : usize,
+    pub tracks : Vec<FrameTrackData>,
     pub delay : f64
 }
 
 pub struct AnimationImportData
 {
-    pub name : String,
+    pub animation_name : String,
     pub frames : Vec<FrameImportData>,
     pub looping : bool
 }
@@ -235,6 +241,7 @@ pub struct SpriteImportData
     pub palette : Option<Vec<Color>>,
     pub images : Vec<ImageImportData>,
     pub animations : Vec<AnimationImportData>,
+    pub default_track_name : String,
     pub spawn_type_name : String,
     pub dest_sheet_package : String,
     pub dest_sheet : String,
@@ -253,9 +260,9 @@ impl SpriteImportData
         let mut ids : Vec<usize> = (0..self.images.len()).collect();
 
         ids.sort_by(|a, b|
-            {
-                self.images[*a].partial_cmp(&self.images[*b]).unwrap_or(Ordering::Equal)
-            });
+        {
+            self.images[*a].partial_cmp(&self.images[*b]).unwrap_or(Ordering::Equal)
+        });
 
         // Find duplicates using this newly sorted ID list
         let mut dups = HashMap::new();
@@ -283,19 +290,19 @@ impl SpriteImportData
         let mut dup_count = 0;
 
         self.images.retain(|_|
+        {
+            let is_dup = dups.contains_key(&(i));
+
+            if is_dup
             {
-                let is_dup = dups.contains_key(&(i));
+                dup_count += 1;
+            }
 
-                if is_dup
-                {
-                    dup_count += 1;
-                }
+            dup_shift[i] += dup_count;
 
-                dup_shift[i] += dup_count;
-
-                i += 1;
-                !is_dup
-            });
+            i += 1;
+            !is_dup
+        });
 
         // Update animation data to point to deduplicated frame IDs, shifted to the left to account
         //  for deleted frames
@@ -303,12 +310,15 @@ impl SpriteImportData
         {
             for frame in &mut animation.frames
             {
-                if dups.contains_key(&frame.index)
+                for track in &mut frame.tracks
                 {
-                    frame.index = dups[&frame.index];
-                }
+                    if dups.contains_key(&track.index)
+                    {
+                        track.index = dups[&track.index];
+                    }
 
-                frame.index -= dup_shift[frame.index];
+                    track.index -= dup_shift[track.index];
+                }
             }
         }
     }
