@@ -14,6 +14,8 @@ extern crate notify;
 extern crate walkdir;
 extern crate path_slash;
 extern crate chrono;
+extern crate sha2;
+extern crate hex;
 extern crate rectangle_pack;
 extern crate asefile;
 
