@@ -75,7 +75,15 @@ pub fn sheet_slice(sheet : &DataHandle<Sheet>, slice : &str, sheet_store : &mut
 
 pub fn cel_image(cel : &AnimationCel, sheet_store : &DataStore<Sheet>, packages : &mut Packages) -> Option<(imgui::Image, Rect<f32>)>
 {
-    sheet_slice_image(&cel.sheet, cel.slice.slice(), sheet_store, packages)
+    if let Some((image, mut rect)) = sheet_slice_image(&cel.sheet, cel.slice.slice(), sheet_store, packages)
+    {
+        rect.x -= cel.x;
+        rect.y -= cel.y;
+
+        return Some((image, rect));
+    }
+
+    None
 }
 
 pub fn cel_sheet_slice(cel : &AnimationCel, sheet_store : &mut DataStore<Sheet>, packages : &mut Packages) -> Option<(DataId<Sheet>, usize)>
