@@ -69,6 +69,17 @@ pub fn round_cursor_screen_pos(ui : &imgui::Ui)
     ]);
 }
 
+pub fn hover_hint(text : &str, ui : &imgui::Ui)
+{
+    ui.same_line();
+    ui.text_disabled("(?)");
+
+    if ui.is_item_hovered()
+    {
+        ui.tooltip_text(text);
+    }
+}
+
 pub fn window_placeholder_text(text : &str, ui : &imgui::Ui)
 {
     let size = ui.calc_text_size(text);
