@@ -1,5 +1,5 @@
 use keeshond::gameloop::GameControl;
-use keeshond_treats::world::Level;
+use keeshond_treats::level::Level;
 
 use crate::editorgui::{Packages};
 use crate::wizard::{Wizard, WizardState};
